﻿package 
{
	import flash.display.Sprite;
	import flash.display.StageQuality;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import org.papervision3d.cameras.CameraType;
	import org.papervision3d.cameras.SpringCamera3D;
	import org.papervision3d.core.math.Number3D;
	import org.papervision3d.materials.special.ParticleMaterial;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.objects.special.ParticleField;
	import org.papervision3d.view.BasicView;
	import org.papervision3d.view.stats.StatsView;
	
	/**
	 * Stardust Particle Engine Demo - X-Wing Engine Rocket Effects
	 * Demo showing how to combine Stardust with Papervision3D to add interactive particle effects to a 3D model
	 * 
	 * Requires Papervision3D 2.1.920 - http://code.google.com/p/papervision3d/
	 * Requires Stardust 1.0.104 - http://code.google.com/p/stardust-particle-engine/
	 * 
	 * Demo by - http://matteley.wordpress.com/
	 * X-Wing model by - http://rockonflash.wordpress.com/2009/07/06/rockonflash-com-home/
	 * X-Wing model available to download from - http://code.google.com/p/papervision3d/downloads/list
	 */
	[SWF(width="800", height="500", backgroundColor="#000000", frameRate="30")]
	public class Main extends BasicView 
	{		

		private var pv3dShip:SpaceShip;
		private var pv3dStars:ParticleField;
		private var pv3dStars2:ParticleField;
		private var pv3dStats:StatsView;
		private var mouseDown:Boolean = false;
		
		public function Main():void 
		{
			super(800, 500, false, false, CameraType.SPRING);
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private function init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			// entry point
			
			stage.quality = StageQuality.MEDIUM;
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			
			initPapervision();
			startRendering();
			
		}
		
		private function initPapervision():void
		{
			pv3dStats = new StatsView(renderer);
			addChild(pv3dStats);
			
			pv3dShip = new SpaceShip();
			scene.addChild(pv3dShip);
			
			var starMaterial:ParticleMaterial = new ParticleMaterial(0xFFFFFF, 1, ParticleMaterial.SHAPE_CIRCLE, 2);
			pv3dStars = new ParticleField(starMaterial, 200, 2, 10000, 1000, 10000);
			scene.addChild(pv3dStars);
			
			pv3dStars2 = new ParticleField(starMaterial, 200, 2, 10000, 1000, 10000);
			pv3dStars2.z = 5000;
			scene.addChild(pv3dStars2);

			SpringCamera3D(camera).mass = 20;
			SpringCamera3D(camera).damping = 10;
			SpringCamera3D(camera).stiffness = 1;
			SpringCamera3D(camera).positionOffset = new Number3D(0, 0, -400);
			SpringCamera3D(camera).lookOffset = new Number3D(0, 0, 0);
			camera.target = pv3dShip;

		}
		
		private function onMouseDown(e:MouseEvent):void
		{
			mouseDown = true;
		}
		
		private function onMouseUp(e:MouseEvent):void
		{
			mouseDown = false;
		}
		
		override protected function onRenderTick(event:Event = null):void
		{
			var xpos:Number = (mouseX - 400);
			var ypos:Number = (mouseY - 250) * -1.2;
			
			pv3dShip.update(mouseDown, (xpos/30));
			SpringCamera3D(camera).positionOffset = new Number3D(xpos, ypos, -400);
			
			if (camera.z - pv3dStars.z > 5000) pv3dStars.z = camera.z + 5000;
			if (camera.z - pv3dStars2.z > 5000) pv3dStars2.z = camera.z + 5000;
			
			renderer.renderScene(scene, camera, viewport);
		}
		
	}
	
}